#include "main.h"

// dynamics and collision objects
static dWorldID world;
static dSpaceID space;
static dBodyID body;	
static dGeomID geom;	
static dMass m;
static dJointGroupID contactgroup;

static MyFeedback feedbacks[1];

static void nearCallback (void *data, dGeomID o1, dGeomID o2)
{
    dBodyID b1 = dGeomGetBody(o1);
    dBodyID b2 = dGeomGetBody(o2);
    dContact contact; 
    contact.surface.mode = dContactBounce | dContactSoftCFM;
    // friction parameter
    contact.surface.mu = dInfinity;
	contact.surface.mu2 = dInfinity;
    // bounce is the amount of "bouncyness".
    contact.surface.bounce = 0.0f;
    // bounce_vel is the minimum incoming velocity to cause a bounce
    contact.surface.bounce_vel = 0.01f;

	contact.surface.soft_cfm = 0.0f;
	
	if (int numc = dCollide (o1,o2,1,&contact.geom,sizeof(dContact))) 
	{
        dJointID c = dJointCreateContact (world,contactgroup,&contact);
        dJointAttach (c,b1,b2);
		
		dJointSetFeedback(c, &feedbacks[0].fb);		
    }           
}

int main (int argc, char **argv)
{
	ofstream outFile;
	CStopWatch sWatch;

	cout.precision(30);

	outFile.open("OutputODETime.csv");
	outFile << "Iteration, Time" << endl;
	
	//Initialise ODE
	dInitODE ();

    //Create world
    world = dWorldCreate ();
    space = dHashSpaceCreate (0);
    dWorldSetGravity (world,0,0,-9.81f);
    dWorldSetCFM (world,1e-5f);
    contactgroup = dJointGroupCreate (0);
    
	//Create sphere
    body = dBodyCreate (world);
    geom = dCreateSphere (space,10.0f);
    dMassSetSphere (&m, 1.0, 10.0f);
    dBodySetMass (body,&m);
    dGeomSetBody (geom,body);

    //Set spheres initial position
    dBodySetPosition (body,0,0,100.0);
    
	int counter = 1000;
	int i = 1;	
	
	//Run simulation
   	while(counter--)
	{
		sWatch.startTimer();
		//Find collisions and add contact joints
		dSpaceCollide (space,0,&nearCallback);
		//Step the simulation
		dWorldQuickStep (world,1.0f/60.0f);  
		//Remove all contact joints
		dJointGroupEmpty (contactgroup);
		sWatch.stopTimer();		

		outFile << i << ", " << sWatch.getElapsedTime() << endl;
		i++;
	}

    //Clean up
    dJointGroupDestroy (contactgroup);
    dSpaceDestroy (space);
    dWorldDestroy (world);
    dCloseODE();

	outFile.close();

    return 0;
}
 